@@ -1,4 +1,4 @@
-/* $Id: dict.c,v 1.16 2002/04/04 03:38:26 kevinv Exp $ */
+/* $Id: dict.c,v 1.20 2003/05/14 14:46:17 utx Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -76,10 +76,11 @@ void dict_res_destroy (dict_res_t *res)
 
 static int dict_write (dict_context_t *context, GString *str) 
 {
-    int res, amt;
-  
+    int res;
+    gsize amt;
+
     res = g_io_channel_write(context->channel, str->str, str->len, &amt);
-    
+
     if (res != G_IO_ERROR_NONE)
 	return -1;
     else
@@ -123,7 +124,12 @@ dict_command_done (dict_command_t *command)
 static int 
 cycle_buffer (dict_context_t *context) 
 {
-    int amt_read, res;
+    gsize amt_read;
+    int res;
+    GIOChannelError chanerr;
+
+    if (!context->channel)
+	return -1;
     
     if (!context->read_cycle_done) {
         if (context->write_ptr > context->read_ptr)
@@ -139,9 +145,14 @@ cycle_buffer (dict_context_t *context)
         
         if (res == G_IO_ERROR_AGAIN)
 	    context->read_cycle_done = TRUE;
-        else if (res != G_IO_ERROR_NONE)
+	else if (res == G_IO_ERROR_UNKNOWN) {
+		chanerr = g_io_channel_error_from_errno(errno);
+		if (chanerr == G_IO_CHANNEL_ERROR_PIPE || G_IO_CHANNEL_ERROR_FAILED)
+			context->channel = NULL;
+		return -1;
+        } else if (res != G_IO_ERROR_NONE)
 	    return -1;
-        
+
         context->write_ptr += amt_read;
         return 0;
     }
@@ -319,7 +330,9 @@ static void data_notify_define (dict_context_t *context, char *line)
             buf = line;
             while (!isspace (*buf)) buf++;
             *buf = '\0';
-            
+	    ++buf;
+           
+	    command->db_full_name = g_strdup (buf); 
             command->db_name = g_strdup (line);
             command->def = g_string_new (NULL);
             command->state = S_DATA;
@@ -763,9 +776,8 @@ int dict_connect (dict_context_t *context)
 
 /* dict_disconnect (context)
  *
- * Opens a connection to the DICT server.
+ * Closes a connection to the DICT server.
  *
- * Returns 0 on success, -1 on connection error
  */
 
 void dict_disconnect (dict_context_t *context) 
